import http from "http";
import fs from "fs";
import path from "path";
import grabHostNames from "../package-shared/utils/grab-host-names";
import apiSendEmailCode from "../package-shared/functions/api/users/api-send-email-code";
import { SendOneTimeCodeEmailResponse } from "../package-shared/types";

type Param = {
    key?: string;
    database?: string;
    email: string;
    temp_code_field_name?: string;
    response?: http.ServerResponse & { [s: string]: any };
    mail_domain?: string;
    mail_username?: string;
    mail_password?: string;
    mail_port?: number;
    sender?: string;
    user_id?: boolean;
    useLocal?: boolean;
    extraCookies?: import("../package-shared/types").CookieObject[];
};

/**
 * # Send Email Code to a User
 */
export default async function sendEmailCode({
    key,
    email,
    database,
    temp_code_field_name,
    mail_domain,
    mail_password,
    mail_username,
    mail_port,
    sender,
    user_id,
    useLocal,
    response,
    extraCookies,
}: Param): Promise<SendOneTimeCodeEmailResponse> {
    const grabedHostNames = grabHostNames();
    const { host, port, scheme } = grabedHostNames;

    const defaultTempLoginFieldName = "temp_login_code";
    const emailLoginTempCodeFieldName = temp_code_field_name
        ? temp_code_field_name
        : defaultTempLoginFieldName;

    const emailHtml = fs.readFileSync(
        path.resolve(__dirname, "../package-shared/html/one-time-code.html"),
        "utf-8"
    );

    /**
     * Check for local DB settings
     *
     * @description Look for local db settings in `.env` file and by pass the http request if available
     */
    const { DSQL_DB_HOST, DSQL_DB_USERNAME, DSQL_DB_PASSWORD, DSQL_DB_NAME } =
        process.env;

    if (
        DSQL_DB_HOST?.match(/./) &&
        DSQL_DB_USERNAME?.match(/./) &&
        DSQL_DB_PASSWORD?.match(/./) &&
        DSQL_DB_NAME?.match(/./) &&
        useLocal
    ) {
        /** @type {import("../package-shared/types").DSQL_DatabaseSchemaType | undefined} */
        let dbSchema:
            | import("../package-shared/types").DSQL_DatabaseSchemaType
            | undefined;

        try {
            const localDbSchemaPath = path.resolve(
                process.cwd(),
                "dsql.schema.json"
            );
            dbSchema = JSON.parse(fs.readFileSync(localDbSchemaPath, "utf8"));
        } catch (error) {}

        return await apiSendEmailCode({
            database: DSQL_DB_NAME,
            email,
            email_login_field: emailLoginTempCodeFieldName,
            html: emailHtml,
            mail_domain,
            mail_password,
            mail_port,
            mail_username,
            sender,
            useLocal,
            response,
            extraCookies,
        });
    } else {
        /**
         * Make https request
         *
         * @description make a request to datasquirel.com
         *
         * @type {import("../package-shared/types").SendOneTimeCodeEmailResponse}
         */
        const httpResponse: import("../package-shared/types").SendOneTimeCodeEmailResponse =
            await new Promise((resolve, reject) => {
                const reqPayload = JSON.stringify({
                    email,
                    database,
                    email_login_field: emailLoginTempCodeFieldName,
                    mail_domain,
                    mail_password,
                    mail_username,
                    mail_port,
                    sender,
                    html: emailHtml,
                });

                const httpsRequest = scheme.request(
                    {
                        method: "POST",
                        headers: {
                            "Content-Type": "application/json",
                            "Content-Length": Buffer.from(reqPayload).length,
                            Authorization:
                                key ||
                                process.env.DSQL_FULL_ACCESS_API_KEY ||
                                process.env.DSQL_API_KEY,
                        },
                        port,
                        hostname: host,
                        path: `/api/user/${
                            user_id || grabedHostNames.user_id
                        }/send-email-code`,
                    },

                    /**
                     * Callback Function
                     *
                     * @description https request callback
                     */
                    (res) => {
                        var str = "";

                        res.on("data", function (chunk) {
                            str += chunk;
                        });

                        res.on("end", function () {
                            resolve(JSON.parse(str));
                        });

                        res.on("error", (err) => {
                            reject(err);
                        });
                    }
                );

                httpsRequest.write(reqPayload);
                httpsRequest.end();
            });

        return httpResponse;
    }
}
