// @ts-check

import https from "node:https";
import path from "path";
import fs from "fs";
import grabHostNames from "../package-shared/utils/grab-host-names";
import apiGet from "../package-shared/functions/api/query/get";
import serializeQuery from "../package-shared/utils/serialize-query";
import { GetReturn } from "../package-shared/types";

type Param = {
    key?: string;
    db?: string;
    query: string;
    queryValues?: string[];
    tableName?: string;
    useLocal?: boolean;
    user_id?: string | number;
};

/**
 * # Make a get request to Datasquirel API
 */
export default async function get({
    key,
    db,
    query,
    queryValues,
    tableName,
    useLocal,
    user_id,
}: Param): Promise<GetReturn> {
    const grabedHostNames = grabHostNames();
    const { host, port, scheme } = grabedHostNames;

    /**
     * Check for local DB settings
     *
     * @description Look for local db settings in `.env` file and by pass the http request if available
     */
    const { DSQL_DB_HOST, DSQL_DB_USERNAME, DSQL_DB_PASSWORD, DSQL_DB_NAME } =
        process.env;

    if (
        DSQL_DB_HOST?.match(/./) &&
        DSQL_DB_USERNAME?.match(/./) &&
        DSQL_DB_PASSWORD?.match(/./) &&
        DSQL_DB_NAME?.match(/./) &&
        useLocal
    ) {
        /** @type {import("../package-shared/types").DSQL_DatabaseSchemaType | undefined} */
        let dbSchema:
            | import("../package-shared/types").DSQL_DatabaseSchemaType
            | undefined;

        try {
            const localDbSchemaPath = path.resolve(
                process.cwd(),
                "dsql.schema.json"
            );
            dbSchema = JSON.parse(fs.readFileSync(localDbSchemaPath, "utf8"));
        } catch (error) {}

        return await apiGet({
            dbFullName: DSQL_DB_NAME,
            query,
            queryValues,
            tableName,
            dbSchema,
            useLocal,
        });
    }

    /**
     * Make https request
     *
     * @description make a request to datasquirel.com
     */
    const httpResponse = await new Promise((resolve, reject) => {
        /** @type {import("../package-shared/types").GetReqQueryObject} */
        const queryObject: import("../package-shared/types").GetReqQueryObject =
            {
                db: process.env.DSQL_API_DB_NAME || String(db),
                query: String(
                    query.replace(/\n|\r|\n\r/g, "").replace(/ {2,}/g, " ")
                ),
                queryValues: queryValues
                    ? JSON.stringify(queryValues)
                    : undefined,
                tableName,
            };

        const queryString = serializeQuery({ ...queryObject });

        let path = `/api/query/${
            user_id || grabedHostNames.user_id
        }/get${queryString}`;

        /** @type {https.RequestOptions} */
        const requestObject: https.RequestOptions = {
            method: "GET",
            headers: {
                "Content-Type": "application/json",
                Authorization:
                    key ||
                    process.env.DSQL_READ_ONLY_API_KEY ||
                    process.env.DSQL_FULL_ACCESS_API_KEY ||
                    process.env.DSQL_API_KEY,
            },
            port,
            hostname: host,
            path,
        };

        scheme
            .request(
                requestObject,

                /**
                 * Callback Function
                 *
                 * @description https request callback
                 */
                (response) => {
                    var str = "";

                    response.on("data", function (chunk) {
                        str += chunk;
                    });

                    response.on("end", function () {
                        try {
                            resolve(JSON.parse(str) as GetReturn);
                        } catch (/** @type {any} */ error: any) {
                            reject({
                                error: error.message,
                                result: str,
                            });
                        }
                    });

                    response.on("error", (err) => {
                        console.log("DSQL get Error,", err.message);
                        resolve(null);
                    });
                }
            )
            .end();
    });

    return httpResponse as GetReturn;
}
